package com.winhc.repal.aspect;


import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.winhc.common.enums.CodeMsg;
import com.winhc.common.exception.CommonException;
import com.winhc.repal.annotation.RolePermission;
import com.winhc.repal.constant.Constant;
import com.winhc.repal.service.RepalRolePermissionService;
import com.winhc.repal.util.RedisUtil;
import com.winhc.repal.util.UserContextUtil;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.*;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;

/**
 * ControllerAspect
 * @author xda
 */
@Aspect
@Component
@Order(Integer.MIN_VALUE)
public class ControllerAspect {

    private static final long TIME_OUT = 800L;

    private static final Logger LOGGER = LoggerFactory.getLogger(ControllerAspect.class);

    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private RepalRolePermissionService repalRolePermissionService;

    @Pointcut("execution(* com.winhc.repal.controller.*Controller.*(..))")
    public void validAspect() {

    }

    @Around("validAspect()")
    public Object aroundExecution(ProceedingJoinPoint joinPoint) throws Throwable {
        long startTime = System.currentTimeMillis();
        StringBuilder mappingRoute = new StringBuilder();
        Object[] params = joinPoint.getArgs();
        Signature signature = joinPoint.getSignature();
        String methodName = "";
        Annotation[] controllerAnnotations = signature.getDeclaringType().getDeclaredAnnotations();
        for (Annotation a : controllerAnnotations){
            if( a instanceof RequestMapping){
                mappingRoute.append(((RequestMapping) a).value().length > 0 ? ((RequestMapping) a).value()[0] : ((RequestMapping) a).path()[0]);
                break;
            }
        }
        MethodSignature methodSignature = (MethodSignature) signature;
        Method targetMethod = methodSignature.getMethod();
        Annotation[] methodAnnotations = targetMethod.getDeclaredAnnotations();
        for (Annotation a : methodAnnotations) {
            if (a instanceof RequestMapping) {
                mappingRoute.append(((RequestMapping) a).value().length > 0 ? ((RequestMapping) a).value()[0] : ((RequestMapping) a).path()[0]);
                methodName = ((RequestMapping) a).method()[0].name();
            }
            if (a instanceof GetMapping) {
                mappingRoute.append(((GetMapping) a).value().length > 0 ? ((GetMapping) a).value()[0] : ((GetMapping) a).path()[0]);
                methodName = "GET";
            }
            if (a instanceof PutMapping) {
                mappingRoute.append(((PutMapping) a).value().length > 0 ? ((PutMapping) a).value()[0] : ((PutMapping) a).path()[0]);
                methodName = "PUT";
            }
            if (a instanceof PostMapping) {
                mappingRoute.append(((PostMapping) a).value().length > 0 ? ((PostMapping) a).value()[0] : ((PostMapping) a).path()[0]);
                methodName = "POST";
            }
            if (a instanceof DeleteMapping) {
                mappingRoute.append(((DeleteMapping) a).value().length > 0 ? ((DeleteMapping) a).value()[0] : ((DeleteMapping) a).path()[0]);
                methodName = "DELETE";
            }
            if (a instanceof RolePermission) {
                // 获取注解的值
                String userKey = Constant.USER_PERMISSION_KEY + UserContextUtil.getUser().getUserId();
                if (redisUtil.hasKey(userKey)) {
                    if (!redisUtil.sIsMember(userKey, ((RolePermission)a).permission())) {
                        LOGGER.error("user permission error, ======>>>>>>userId:[{}], ======>>>>>>permissionCode:[{}]", UserContextUtil.getUser().getUserId(), ((RolePermission)a).permission());
                        throw new CommonException(CodeMsg.FAILED, "没有相关权限");
                    }
                } else {
                    if (!repalRolePermissionService.checkUserPermission(UserContextUtil.getUser().getUserId(), ((RolePermission)a).permission().getCode())) {
                        LOGGER.error("user permission error, ======>>>>>>userId:[{}], ======>>>>>>permissionCode:[{}]", UserContextUtil.getUser().getUserId(), ((RolePermission)a).permission());
                        throw new CommonException(CodeMsg.FAILED, "没有相关权限");
                    }
                }
            }
        }
        String requestInfo = "======>>>>>>requestInfo:" + mappingRoute + ",method:[" + methodName + "]" + "(" + StrUtil.repeatAndJoin("{}", params.length, ",") + ")";
        LOGGER.info(requestInfo, params);
        Object resp = null;
        try{
            resp = joinPoint.proceed(params);
        }catch(Throwable e){
            LOGGER.error( String.format("joinPoint.proceed 发生异常,参数 %s ", JSON.toJSONString(params)), e);
            throw e;
        }finally{
            String responseInfo = "======>>>>>>responseInfo:{}";
            long consumedTime = System.currentTimeMillis() - startTime;
            if (consumedTime >= TIME_OUT) {
                LOGGER.warn("=====>>>>>请求共花费{}ms.", consumedTime);
            }
            LOGGER.info(responseInfo, resp);
        }
        return resp;
    }

}
